@@ -1020,6 +1020,17 @@ typedef struct
     NV_STATUS       rmStatus;                    // OUT
 } UVM_POPULATE_PAGEABLE_PARAMS;
 
+//
+// UvmValidateVaRange
+//
+#define UVM_VALIDATE_VA_RANGE                                         UVM_IOCTL_BASE(72)
+typedef struct
+{
+    NvU64           base      NV_ALIGN_BYTES(8); // IN
+    NvU64           length    NV_ALIGN_BYTES(8); // IN
+    NV_STATUS       rmStatus;                    // OUT
+} UVM_VALIDATE_VA_RANGE_PARAMS;
+
 //
 // Temporary ioctls which should be removed before UVM 8 release
 // Number backwards from 2047 - highest custom ioctl function number
