@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 1999-2018 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 1999-2019 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -185,9 +185,92 @@ nvidia_vma_access(
 }
 #endif
 
+static vm_fault_t nvidia_fault(
+#if !defined(NV_VM_OPS_FAULT_REMOVED_VMA_ARG)
+    struct vm_area_struct *vma,
+#endif
+    struct vm_fault *vmf
+)
+{
+#if defined(NV_VM_OPS_FAULT_REMOVED_VMA_ARG)
+    struct vm_area_struct *vma = vmf->vma;
+#endif
+    nv_file_private_t *nvfp = NV_GET_FILE_PRIVATE(NV_VMA_FILE(vma));
+    nv_linux_state_t *nvl = nvfp->nvptr;
+    nv_state_t *nv = NV_STATE_PTR(nvl);
+    vm_fault_t ret = VM_FAULT_NOPAGE;
+
+    NvU64 page;
+    NvU64 num_pages = NV_VMA_SIZE(vma) >> PAGE_SHIFT;
+    NvU64 pfn_start =
+        (nvfp->mmap_context.mmap_start >> PAGE_SHIFT) + vma->vm_pgoff;
+
+    // Mapping revocation is only supported for GPU mappings.
+    if (NV_IS_CTL_DEVICE(nv))
+    {
+        return VM_FAULT_SIGBUS;
+    }
+
+    down(&nvl->mmap_lock);
+
+    // Wake up the GPU if it is not currently safe to mmap.
+    if (!nvl->safe_to_mmap)
+    {
+        NV_STATUS status;
+
+        if (!nvl->gpu_wakeup_callback_needed)
+        {
+            // GPU wakeup callback already scheduled.
+            up(&nvl->mmap_lock);
+            return VM_FAULT_NOPAGE;
+        }
+
+        /*
+         * GPU wakeup cannot be completed directly in the fault handler due to the
+         * inability to take the GPU lock while mmap_sem is held.
+         */
+        status = rm_schedule_gpu_wakeup(nvl->sp[NV_DEV_STACK_GPU_WAKEUP], nv);
+        if (status != NV_OK)
+        {
+            nv_printf(NV_DBG_ERRORS,
+                      "NVRM: VM: rm_schedule_gpu_wakeup failed: %x\n", status);
+            up(&nvl->mmap_lock);
+            return VM_FAULT_SIGBUS;
+        }
+        // Ensure that we do not schedule duplicate GPU wakeup callbacks.
+        nvl->gpu_wakeup_callback_needed = FALSE;
+
+        up(&nvl->mmap_lock);
+        return VM_FAULT_NOPAGE;
+    }
+
+    // Safe to mmap, map all pages in this VMA.
+    for (page = 0; page < num_pages; page++)
+    {
+        NvU64 virt_addr = vma->vm_start + (page << PAGE_SHIFT);
+        NvU64 pfn = pfn_start + page;
+
+        ret = nv_insert_pfn(vma, virt_addr, pfn,
+                            nvfp->mmap_context.remap_prot_extra);
+        if (ret != VM_FAULT_NOPAGE)
+        {
+            nv_printf(NV_DBG_ERRORS,
+                      "NVRM: VM: nv_insert_pfn failed: %x\n", ret);
+            break;
+        }
+
+        nvl->all_mappings_revoked = FALSE;
+    }
+
+    up(&nvl->mmap_lock);
+
+    return ret;
+}
+
 static struct vm_operations_struct nv_vm_ops = {
     .open   = nvidia_vma_open,
     .close  = nvidia_vma_release,
+    .fault  = nvidia_fault,
 #if defined(NV_VM_OPERATIONS_STRUCT_HAS_ACCESS)
     .access = nvidia_vma_access,
 #endif
@@ -311,7 +394,6 @@ int static nvidia_mmap_sysmem(
         nv_speculation_barrier();
 #endif
 
-#if defined(NV_VM_INSERT_PAGE_PRESENT)
 #if defined(NV_VGPU_KVM_BUILD)
         if (NV_ALLOC_MAPPING_GUEST(at->flags))
         {
@@ -321,14 +403,11 @@ int static nvidia_mmap_sysmem(
         else
 #endif
         {
-            ret = NV_VM_INSERT_PAGE(vma, start,
-                                    NV_GET_PAGE_STRUCT(at->page_table[j]->phys_addr));
+            ret = vm_insert_page(vma, start,
+                                 NV_GET_PAGE_STRUCT(at->page_table[j]->phys_addr));
         }
+
         if (ret)
-#else
-        if (nv_remap_page_range(vma, start, at->page_table[j]->phys_addr,
-                                PAGE_SIZE, vma->vm_page_prot) != 0)
-#endif
         {
             NV_ATOMIC_DEC(at->usage_count);
             return -EAGAIN;
@@ -339,7 +418,7 @@ int static nvidia_mmap_sysmem(
     return ret;
 }
 
-#if defined(NVCPU_PPC64LE) && defined(NV_VM_INSERT_PAGE_PRESENT)
+#if defined(NVCPU_PPC64LE)
 static int nvidia_mmap_numa(
     struct vm_area_struct *vma,
     const nv_alloc_mapping_context_t *mmap_context)
@@ -383,6 +462,7 @@ int nvidia_mmap_helper(
     NvU32 prot = 0;
     int ret;
     const nv_alloc_mapping_context_t *mmap_context = &nvfp->mmap_context;
+    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
     NV_STATUS status;
 
     if (nvfp == NULL)
@@ -400,12 +480,11 @@ int nvidia_mmap_helper(
 
     NV_PRINT_VMA(NV_DBG_MEMINFO, vma);
 
-    status = nv_check_pci_config_space_and_gpu_state(sp, nv, TRUE,
-                                                     FALSE, NV_MAY_SLEEP());
+    status = nv_check_gpu_state(nv);
     if (status != NV_OK)
     {
-        nv_printf(NV_DBG_INFO,
-            "NVRM: GPU is lost, skipping nvidia_mmap_helper\n");
+        NV_DEV_PRINTF(NV_DBG_INFO, nv,
+            "GPU is lost, skipping nvidia_mmap_helper\n");
         return status;
     }
 
@@ -457,33 +536,41 @@ int nvidia_mmap_helper(
             }
         }
 
-#if defined(NVCPU_PPC64LE) && defined(NV_VM_INSERT_PAGE_PRESENT)
-        // 
-        // This path is similar to the sysmem mapping code. 
-        // TODO: Refactor is needed as part of bug#2001704.
-        // Use pfn_valid to determine whether the physical address has
-        // backing struct page. This is used to isolate P8 from P9. 
-        //
-        if (!IS_REG_OFFSET(nv, access_start, access_len) && 
-            (pfn_valid(PFN_DOWN(mmap_start))))
+        down(&nvl->mmap_lock);
+        if (nvl->safe_to_mmap)
         {
-            ret = nvidia_mmap_numa(vma, mmap_context);
-            if (ret)
+            nvl->all_mappings_revoked = FALSE;
+#if defined(NVCPU_PPC64LE)
+            //
+            // This path is similar to the sysmem mapping code.
+            // TODO: Refactor is needed as part of bug#2001704.
+            // Use pfn_valid to determine whether the physical address has
+            // backing struct page. This is used to isolate P8 from P9.
+            //
+            if (!IS_REG_OFFSET(nv, access_start, access_len) &&
+                (pfn_valid(PFN_DOWN(mmap_start))))
             {
-                return ret;
+                ret = nvidia_mmap_numa(vma, mmap_context);
+                if (ret)
+                {
+                    up(&nvl->mmap_lock);
+                    return ret;
+                }
             }
-        }
-        else
+            else
 #endif
-        {
-            if (nv_io_remap_page_range(vma, mmap_start, mmap_length,
-                    remap_prot_extra) != 0)
             {
-                return -EAGAIN;
+                if (nv_io_remap_page_range(vma, mmap_start, mmap_length,
+                        remap_prot_extra) != 0)
+                {
+                    up(&nvl->mmap_lock);
+                    return -EAGAIN;
+                }
             }
         }
+        up(&nvl->mmap_lock);
 
-        vma->vm_flags |= VM_IO;
+        vma->vm_flags |= VM_IO | VM_PFNMAP | VM_DONTEXPAND;
     }
     else
     {
@@ -596,3 +683,92 @@ int nvidia_mmap(
 
     return status;
 }
+
+void
+nv_revoke_gpu_mappings_locked(
+    nv_state_t *nv
+)
+{
+    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+    nv_file_private_t *nvfp;
+
+    /* Revoke all mappings for every open file */
+    list_for_each_entry (nvfp, &nvl->open_files, entry)
+    {
+        unmap_mapping_range(&nvfp->mapping, 0, ~0, 1);
+    }
+
+    nvl->all_mappings_revoked = TRUE;
+}
+
+NV_STATUS NV_API_CALL nv_revoke_gpu_mappings(
+    nv_state_t *nv
+)
+{
+    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+
+    // Mapping revocation is only supported for GPU mappings.
+    if (NV_IS_CTL_DEVICE(nv))
+    {
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    down(&nvl->mmap_lock);
+
+    nv_revoke_gpu_mappings_locked(nv);
+
+    up(&nvl->mmap_lock);
+
+    return NV_OK;
+}
+
+void NV_API_CALL nv_acquire_mmap_lock(
+    nv_state_t *nv
+)
+{
+    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+    down(&nvl->mmap_lock);
+}
+
+void NV_API_CALL nv_release_mmap_lock(
+    nv_state_t *nv
+)
+{
+    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+    up(&nvl->mmap_lock);
+}
+
+NvBool NV_API_CALL nv_get_all_mappings_revoked_locked(
+    nv_state_t *nv
+)
+{
+    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+
+    // Caller must hold nvl->mmap_lock for all decisions based on this
+    return nvl->all_mappings_revoked;
+}
+
+void NV_API_CALL nv_set_safe_to_mmap_locked(
+    nv_state_t *nv,
+    NvBool safe_to_mmap
+)
+{
+    nv_linux_state_t *nvl = NV_GET_NVL_FROM_NV_STATE(nv);
+
+    // Caller must hold nvl->mmap_lock
+
+    /*
+     * If nvl->safe_to_mmap is transitioning from TRUE to FALSE, we expect to
+     * need to schedule a GPU wakeup callback when we fault.
+     *
+     * nvl->gpu_wakeup_callback_needed will be set to FALSE in nvidia_fault()
+     * after scheduling the GPU wakeup callback, preventing us from scheduling
+     * duplicates.
+     */
+    if (!safe_to_mmap && nvl->safe_to_mmap)
+    {
+        nvl->gpu_wakeup_callback_needed = TRUE;
+    }
+
+    nvl->safe_to_mmap = safe_to_mmap;
+}
